import type { HeroType } from "../types";
import type { VillageState } from "../village";


const LEVELS = [
    0,
    10,
    25,
    50,
    100,
    250,
    500,
    1000,
    2500,
    5000,
    10000,
];


export function getHero(V: VillageState, heroId: number) {
    const hero = V.heroes.find(h => h.id === heroId);
    if (!hero) {
        throw new Error(`Cannot find hero with id "${heroId}"`);
    }
    return hero;
}


export function getMaxHealth(hero: HeroType): number {
    return 100.0 + (hero.level - 1) * 5.0;
}


export function hurt(hero: HeroType, damage: number) {
    hero.health -= damage;

    if (hero.health <= 0) {
        hero.health = 0.0;

        // Do something to show that the hero is dead.
    }
}


export function heal(hero: HeroType, health: number) {
    hero.health -= health;

    const maxHealth = getMaxHealth(hero);
    if (hero.health > maxHealth) {
        hero.health = maxHealth;
    }
}


export function gainExperience(hero: HeroType, xp: number) {
    hero.experience += xp;

    if (hero.experience >= LEVELS[hero.level]) {
        hero.experience -= LEVELS[hero.level];

        hero.level++;
        hero.health = getMaxHealth(hero);
    }
}


export function getExperienceNeeded(hero: HeroType) {
    return LEVELS[hero.level];
}


export default {
    getHero,
    getExperienceNeeded,
    getMaxHealth,
    heal,
    hurt,
    gainExperience,
};
